
#include <stdint.h>
#include <stdio.h>
#include <string.h>
#include <stdlib.h>

#include "nrf_drv_gpiote.h"
#include "nrf_drv_ppi.h"
#include "nrf_drv_timer.h"
#include "app.h"
#include "nrf_log.h"
#include "nrf_log_ctrl.h"
#include "nrf_log_default_backends.h"
#include "nrf_delay.h"

#define BG51_PULSES_SENSOR_PIN  NRF_GPIO_PIN_MAP(0,31)


static const nrf_drv_timer_t m_timer1 = NRF_DRV_TIMER_INSTANCE(1);
static const nrf_drv_timer_t m_timer2 = NRF_DRV_TIMER_INSTANCE(2);
test_data dosimeter_data;

static void pulses_gpiote_init(void); 
static void pulses_timer1_init(void); 
static void pulses_timer2_init(void); 
static void pulses_ppi_init(void); 
static void gpiote_in_event_dummy_handler(nrf_drv_gpiote_pin_t pin, nrf_gpiote_polarity_t action) {}
static void pulses_start(void);
static void timer2_event_handler(nrf_timer_event_t event_type, void *p_context) {}
static void pulses_stop(void);
static void timer1_event_handler(nrf_timer_event_t event_type, void *p_context);


static uint8_t cur_pulse_count = 0;
static uint16_t timeSinceStart = 0;                                     // in seconds
static float lastWaterConsumption = 0;
static uint32_t lasttotalPulseCount = 0;
static uint32_t lastPulseCount = 0;
uint32_t pulseCount = 0;


static nrf_ppi_channel_t m_ppi_channel1;
static nrf_ppi_channel_t m_ppi_channel2;

void test_init(void) 
{
    pulses_gpiote_init();
    pulses_timer1_init();
    pulses_timer2_init();
    pulses_ppi_init();

   nrf_drv_gpiote_in_event_enable(BG51_PULSES_SENSOR_PIN, true);
   nrf_drv_timer_enable(&m_timer1);
   nrf_drv_timer_enable(&m_timer2);

  //  test_load_settings();

}


void process_timer_event(void)
{

       // NRF_LOG_INFO("cur_pulse_count is %d",cur_pulse_count);

     

    //nrf_drv_timer_clear(&m_timer1);
    //nrf_drv_timer_clear(&m_timer2);
     pulseCount = cur_pulse_count;
    dosimeter_data.pulses = cur_pulse_count; 

    if ((pulseCount == 0 ) && (lastPulseCount != 0)) {
          pulses_stop();
        /* pulses  stop */
    }

    /* start water flow */
    if ((pulseCount > 0) && (lastPulseCount == 0)) {
        //pulses start
        pulses_start();
    }

    if (dosimeter_data.pulses) 
    {
   //     timeSinceStart += PULSES_TIMER1_INTERVAL / 1000;
   //     dosimeter_data.rem_per_min  = pulses_calc_per_min(pulseCount);
  	//id_send_data.rem_per_min = dosimeter_data.rem_per_min;
  	//doc_send_data.rem_per_min = dosimeter_data.rem_per_min;
         lasttotalPulseCount += pulseCount;

   //     // NRF_LOG_INFO("Pulse count: %d in time %d and water consumption"NRF_LOG_FLOAT_MARKER" and temperature %d", pulseCount, timeSinceStart, NRF_LOG_FLOAT(lastWaterConsumption), shower.temperature);

        nrf_drv_timer_clear(&m_timer2);

        //dosimeter_data.duration += PULSES_TIMER1_INTERVAL / 1000;
        //dosimeter_data.total_duration = timeSinceStart;
        //id_send_data.time = timeSinceStart;
        dosimeter_data.totalPulse = lasttotalPulseCount;
        //dosimeter_data.totalPulse_1 = lasttotalPulseCount;
        //dosimeter_data.num_of_hours = dosimeter_data.duration / SECONDS_PER_HOUR;
        //dosimeter_data.num_of_min = dosimeter_data.duration / 60;
        //dosimeter_data.num_of_sec = dosimeter_data.duration;


        //if (dosimeter_data.total_duration > (60 * 60))
        //{
        //  dosimeter_data.rem_per_hr  = pulses_calc_per_hr(lasttotalPulseCount);
        //  dosimeter_data.num_of_hours = dosimeter_data.duration / SECONDS_PER_HOUR;
        //}
    }

        //sprintf((char *)buffer,"%d",pulseCount);
        //       NRF_LOG_INFO("buffer is %s",buffer);

    /* Save pulse count */
    lastPulseCount = pulseCount;
}



static void pulses_start(void)
{
    uint8_t i = 0;
    //NRF_LOG_INFO("PULSES DETECTED!");
    
    
    /* Start New Session */
    dosimeter_data.isNewSession = 1;
    dosimeter_data.pulses = 1;

}

static void pulses_stop(void)
{
    uint8_t i = 0;
    //NRF_LOG_INFO("PULSES STOPPED!");
    
    nrf_drv_timer_clear(&m_timer1);
    nrf_drv_timer_clear(&m_timer2);
    /* Start New Session */
    dosimeter_data.pulses = 0;
    
}



static void pulses_gpiote_init(void) 
{
    ret_code_t err_code;
    if (!nrf_drv_gpiote_is_init()) {
         err_code = nrf_drv_gpiote_init();
         NRF_LOG_INFO("Initializing pulses measurement %d", err_code);
         APP_ERROR_CHECK(err_code);
    }

    nrf_drv_gpiote_in_config_t config = GPIOTE_CONFIG_IN_SENSE_LOTOHI(true);
    config.pull = NRF_GPIO_PIN_NOPULL;

    if (!nrf_drv_gpiote_in_is_set(BG51_PULSES_SENSOR_PIN)) {
        err_code = nrf_drv_gpiote_in_init(BG51_PULSES_SENSOR_PIN, &config, gpiote_in_event_dummy_handler);
        APP_ERROR_CHECK(err_code);
    }
}


static void pulses_timer1_init(void) 
{
    nrf_drv_timer_config_t timer_cfg = NRF_DRV_TIMER_DEFAULT_CONFIG;
    timer_cfg.frequency = NRF_TIMER_FREQ_31250Hz;
    timer_cfg.bit_width = NRF_TIMER_BIT_WIDTH_24;
    ret_code_t err_code = nrf_drv_timer_init(&m_timer1, &timer_cfg, timer1_event_handler);
    APP_ERROR_CHECK(err_code);

    nrf_drv_timer_extended_compare(&m_timer1,
                                    NRF_TIMER_CC_CHANNEL0,
                                    nrf_drv_timer_ms_to_ticks(&m_timer1, PULSES_TIMER1_INTERVAL),
                                    NRF_TIMER_SHORT_COMPARE0_CLEAR_MASK, true);
}

/**@brief PPI init
 *
 */
static void pulses_ppi_init(void) 
{
    // Stop counter ppi
    uint32_t err_code = NRF_SUCCESS;

    err_code = nrf_drv_ppi_init();
    APP_ERROR_CHECK(err_code);

    // Config timer 1 to stop timer2
    err_code = nrf_drv_ppi_channel_alloc(&m_ppi_channel1);
    APP_ERROR_CHECK(err_code);
    err_code = nrf_drv_ppi_channel_assign(m_ppi_channel1,
                                          nrf_drv_timer_event_address_get(&m_timer1, NRF_TIMER_EVENT_COMPARE0),
                                          nrf_drv_timer_task_address_get(&m_timer2, NRF_TIMER_TASK_CAPTURE0));
    APP_ERROR_CHECK(err_code);

    // Config timer 2 to count on rising edge of the water flow measurement pin
    err_code = nrf_drv_ppi_channel_alloc(&m_ppi_channel2);
    APP_ERROR_CHECK(err_code);
    err_code = nrf_drv_ppi_channel_assign(m_ppi_channel2,
                                          nrf_drv_gpiote_in_event_addr_get(BG51_PULSES_SENSOR_PIN),
                                          nrf_drv_timer_task_address_get(&m_timer2, NRF_TIMER_TASK_COUNT));
    APP_ERROR_CHECK(err_code);

    
    err_code = nrf_drv_ppi_channel_fork_assign(m_ppi_channel1,nrf_drv_timer_task_address_get(&m_timer2, NRF_TIMER_TASK_CLEAR));  // Clear counter timer using PPI to make sure this is not delayed by BLE interrupt
    APP_ERROR_CHECK(err_code);

    err_code = nrf_drv_ppi_channel_enable(m_ppi_channel1);
    APP_ERROR_CHECK(err_code);

    err_code = nrf_drv_ppi_channel_enable(m_ppi_channel2);
    APP_ERROR_CHECK(err_code);
}

void pulses_timer2_init(void) 
{
    nrf_drv_timer_config_t timer_cfg = NRF_DRV_TIMER_DEFAULT_CONFIG;
    timer_cfg.mode = NRF_TIMER_MODE_COUNTER;
    timer_cfg.bit_width = NRF_TIMER_BIT_WIDTH_32;
    ret_code_t err_code = nrf_drv_timer_init(&m_timer2, &timer_cfg, timer2_event_handler);
    APP_ERROR_CHECK(err_code);
}

static void timer1_event_handler(nrf_timer_event_t event_type, void *p_context)
{
    float rem_val = 0.0;

   if (event_type == NRF_TIMER_EVENT_COMPARE0) {
        

        /* Capture Pulse Count to avoid extra counts later*/
        cur_pulse_count = nrf_drv_timer_capture_get(&m_timer2, NRF_TIMER_CC_CHANNEL0);
        //test_task_queue_event(TEST_TASK_EVT_TIMER);
        cur_pulse_count = 15;
        NRF_LOG_INFO("pulse: %d",cur_pulse_count);
        rem_val = 20; // 5 * (cur_pulse_count * PULSES_CONSTANT);

        write_data_in_file(cur_pulse_count,rem_val);

        cur_pulse_count=0;

    }

  
}


